/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package conversion;


/**
 *
 * @author hansolo
 */
public class Unit
{
    private UnitDescriptor baseUnit;
    private String unitString;
    private String unitName;
    private double factor;
    private double offset;
    private UnitType unitType;
    
    public Unit(final UnitType UNIT_TYPE)
    {        
        this.unitType = UNIT_TYPE;
        this.baseUnit = Globals.INSTANCE.BASE_UNITS.get(UNIT_TYPE);
        this.unitString = Globals.INSTANCE.BASE_UNITS.get(UNIT_TYPE).getUnitString();
        this.unitName = Globals.INSTANCE.BASE_UNITS.get(UNIT_TYPE).getUnitName();
        this.factor = Globals.INSTANCE.BASE_UNITS.get(UNIT_TYPE).getFactor();
        this.offset = Globals.INSTANCE.BASE_UNITS.get(UNIT_TYPE).getOffset();
    }

    public UnitType getUnitType()
    {
        return this.unitType;
    }

    public void setType(final UnitType UNIT_TYPE)
    {
        this.unitType = UNIT_TYPE;
        setBaseUnit(Globals.INSTANCE.BASE_UNITS.get(UNIT_TYPE));
    }

    public UnitDescriptor getBaseUnit()
    {
        return this.baseUnit;
    }

    public void setBaseUnit(final UnitDescriptor BASE_UNIT)
    {
        if (BASE_UNIT.getUnitType() == getUnitType())
        {
            this.baseUnit = BASE_UNIT;
        }
    }

    public double getFactor()
    {
        return this.factor;
    }

    public double getOffset()
    {
        return this.offset;
    }

    public double convertTo(final double VALUE, final UnitDescriptor UNIT)
    {
        if (UNIT.getUnitType() != getUnitType())
        {
            throw new IllegalArgumentException("units have to be of the same type");
        }                
        return ((((VALUE + baseUnit.getOffset()) * baseUnit.getFactor()) + offset) * factor) / UNIT.getFactor() - UNIT.getOffset();
    }

    public double convertToBaseUnit(final double VALUE, final UnitDescriptor UNIT)
    {
        return ((((VALUE + UNIT.getOffset()) * UNIT.getFactor()) + offset) * factor) / baseUnit.getFactor() - baseUnit.getOffset();
    }

    public String getUnitName()
    {
        return this.unitName;
    }

    public String getUnitString()
    {
        return this.unitString;
    }
    
    public java.util.regex.Pattern getPattern()
    {
        final StringBuilder PATTERN_BUILDER = new StringBuilder();
        PATTERN_BUILDER.append("^([-+]?\\d*\\.?\\d*)\\s?(");        

        for (UnitDescriptor unit : UnitDescriptor.values())
        {
            PATTERN_BUILDER.append(unit.getUnitString().replace("*", "\\*")).append("|");
        }

        PATTERN_BUILDER.deleteCharAt(PATTERN_BUILDER.length() - 1);

        //PATTERN_BUILDER.append("){1}$");
        PATTERN_BUILDER.append(")?$");

        return java.util.regex.Pattern.compile(PATTERN_BUILDER.toString());
    }

    public java.util.EnumMap<UnitType, java.util.ArrayList<UnitDescriptor>> getUnitTypes()
    {
        final java.util.EnumMap<UnitType, java.util.ArrayList<UnitDescriptor>> UNIT_TYPES = new java.util.EnumMap<UnitType, java.util.ArrayList<UnitDescriptor>>(UnitType.class);
        final java.util.ArrayList<UnitType> TYPE_LIST = new java.util.ArrayList<UnitType>(UnitType.values().length);
        TYPE_LIST.addAll(java.util.Arrays.asList(UnitType.values()));
        for (UnitType currentType : TYPE_LIST)
        {
            UNIT_TYPES.put(currentType, new java.util.ArrayList<UnitDescriptor>());
        }

        for (UnitDescriptor unit : UnitDescriptor.values())
        {
            UNIT_TYPES.get(unit.getUnitType()).add(unit);
        }

        return UNIT_TYPES;
    }

    @Override
    public String toString()
    {
        return getUnitType().toString();
    }
}
