package conversion;


public enum UnitDescriptor
{
    // Length
    KILOMETER(UnitType.LENGTH, "km", "KiloMeter", 1000.0, 0.0),
    HECTOMETER(UnitType.LENGTH, "hm", "HectoMeter", 100, 0.0),
    METER(UnitType.LENGTH, "m", "Meter", 1.0, 0.0),
    DECIMETER(UnitType.LENGTH, "dm", "DeciMeter", 0.1, 0.0),
    CENTIMETER(UnitType.LENGTH, "cm", "CentiMeter", 0.01, 0.0),
    MILLIMETER(UnitType.LENGTH, "mm", "MilliMeter", 0.0010, 0.0),
    MIKROMETER(UnitType.LENGTH, "\u00b5m", "MikroMeter", 1.0E-6, 0.0),
    NANOMETER(UnitType.LENGTH, "nm", "NanoMeter", 1.0E-9, 0.0),
    ANGSTROM(UnitType.LENGTH, "\u00c5", "Angstrom", 1.0E-10, 0.0),
    PICOMETER(UnitType.LENGTH, "pm", "PicoMeter", 1.0E-12, 0.0),
    FEMTOMETER(UnitType.LENGTH, "fm", "FemtoMeter", 1.0E-15, 0.0),
    INCHES(UnitType.LENGTH, "in", "Inches", 0.0254, 0.0),
    MILES(UnitType.LENGTH, "miles", "Miles", 1609.344, 0.0),
    FEET(UnitType.LENGTH, "ft", "Feet", 0.3048, 0.0),
    // Mass
    TON(UnitType.MASS, "t", "Ton", 1.0E3, 0.0),
    KILOGRAM(UnitType.MASS, "kg", "KiloGram", 1.0, 0.0),
    GRAM(UnitType.MASS, "g", "Gram", 1.0E-3, 0.0),
    MILLIGRAM(UnitType.MASS, "mg", "MilliGram", 1.0E-6, 0.0),
    MIKROGRAM(UnitType.MASS, "µg", "MikroGram", 1.0E-6, 0.0),
    NANOGRAM(UnitType.MASS, "ng", "NanoGram", 1.0E-9, 0.0),
    PICOGRAM(UnitType.MASS, "pg", "PicoGram", 1.0E-12, 0.0),
    FEMTOGRAM(UnitType.MASS, "fg", "FemtoGram", 1.0E-15, 0.0),
    // Time
    DAY(UnitType.TIME, "d", "Day", 86400, 0.0),
    HOUR(UnitType.TIME, "h", "Hour", 3600, 0.0),
    MINUTE(UnitType.TIME, "m", "Minute", 60, 0.0),
    SECOND(UnitType.TIME, "s", "Second", 1.0, 0.0),
    MILLISECOND(UnitType.TIME, "ms", "MilliSecond", 1E-3, 0.0),
    MICROSECOND(UnitType.TIME, "µs", "MicroSecond", 1E-6, 0.0),
    NANOSECOND(UnitType.TIME, "ns", "NanoSecond", 1E-9, 0.0),
    // Area
    SQUARE_KILOMETER(UnitType.AREA, "km²", "SquareKiloMeter", 1.0E6, 0.0),
    SQUARE_METER(UnitType.AREA, "m²", "Meter", 1.0, 0.0),
    SQUARE_CENTIMETER(UnitType.AREA, "cm²", "SquareCentiMeter", 1.0E-4, 0.0),
    SQUARE_MILLIMETER(UnitType.AREA, "mm²", "SquareMilliMeter", 1.0E-6, 0.0),
    SQUARE_MIKROMETER(UnitType.AREA, "µm²", "SquareMikroMeter", 1.0E-12, 0.0),
    SQUARE_NANOMETER(UnitType.AREA, "nm²", "SquareNanoMeter", 1.0E-18, 0.0),
    SQUARE_ANGSTROM(UnitType.AREA, "Å²", "SquareAngstrom", 1.0E-20, 0.0),
    SQUARE_PICOMETER(UnitType.AREA, "pm²", "SquarePicoMeter", 1.0E-24, 0.0),
    SQUARE_FEMTOMETER(UnitType.AREA, "fm²", "SquareFemtoMeter", 1.0E-30, 0.0),
    HECTARES(UnitType.AREA, "ha", "Hectares", 1.0E5, 0.0),
    ARES(UnitType.AREA, "a", "Ares", 100, 0.0),
    // Temperature
    KELVIN(UnitType.TEMPERATURE, "K", "Kelvin", 1.0, 0.0),
    CELSIUS(UnitType.TEMPERATURE, "°C", "Celsius", 1.0, 273.15),
    FAHRENHEIT(UnitType.TEMPERATURE, "°F", "Fahrenheit", 5.0/9.0, 459.67),
    // Angle
    RADIAN(UnitType.ANGLE, "rad", "Radian", 1.0, 0),
    DEGREE(UnitType.ANGLE, "deg", "Degree", (Math.PI / 180.0), 0),
    // Volume
    CUBIC_MILLIMETER(UnitType.VOLUME, "mm³", "CubicMillimeter", 1.0E-9, 0.0),
    MILLILITER(UnitType.VOLUME, "ml", "MilliLiter", 1.0E-6, 0.0),
    LITER(UnitType.VOLUME, "l", "Liter", 1.0E-3, 0.0),
    CUBIC_METER(UnitType.VOLUME, "m³", "CubicMeter", 1.0E0, 0.0),
    // Voltage
    MILLIVOLT(UnitType.VOLTAGE, "mV", "MilliVolt", 1.0E-3, 0.0),
    VOLT(UnitType.VOLTAGE, "V", "Volt", 1.0E0, 0.0),
    KILOVOLT(UnitType.VOLTAGE, "kV", "KiloVolt", 1.0E3, 0.0),
    // Current
    PICOAMPERE(UnitType.CURRENT, "pA", "PicoAmpere", 1.0E-12, 0.0),
    NANOAMPERE(UnitType.CURRENT, "nA", "NanoAmpere", 1.0E-9, 0.0),
    MICROAMPERE(UnitType.CURRENT, "µA", "MicroAmpere", 1.0E-6, 0.0),
    MILLIAMPERE(UnitType.CURRENT, "mA", "MilliAmpere", 1.0E-3, 0.0),
    AMPERE(UnitType.CURRENT, "A", "Ampere", 1.0E0, 0.0),
    // Speed
    MILLIMETER_PER_SECOND(UnitType.SPEED, "mm/s", "Millimeter per second", 1.0E-3, 0.0),
    METER_PER_SECOND(UnitType.SPEED, "m/s", "Meter per second", 1.0E0, 0.0),
    KILOMETER_PER_HOUR(UnitType.SPEED, "km/h", "KiloMeter per hour", 1.0 / 3.6, 0.0),
    // TemperatureGradient
    KELVIN_PER_SECOND(UnitType.TEMPERATURE_GRADIENT, "K/s", "Kelvin per second", 1.0, 0.0),
    KELVIN_PER_MINUTE(UnitType.TEMPERATURE_GRADIENT, "K/min", "Kelvin per minute", 1.0 / 60.0, 0.0),
    KEVLIN_PER_HOUR(UnitType.TEMPERATURE_GRADIENT, "K/h", "Kelvin per hour", 1.0 / 3600.0, 0.0),
    // ElectricCharge
    ELEMENTARY_CHARGE(UnitType.ELECTRIC_CHARGE, "e-", "Elementary charge", 1.6022E-19, 0.0),
    PICOCOULOMB(UnitType.ELECTRIC_CHARGE, "pC", "PicoCoulomb", 1.0E-12, 0.0),
    NANOCOULOMB(UnitType.ELECTRIC_CHARGE, "nC", "NanoCoulomb", 1.0E-9, 0.0),
    MICROCOULOMB(UnitType.ELECTRIC_CHARGE, "µC", "MicroCoulomb", 1.0E-6, 0.0),
    MILLICOULOMB(UnitType.ELECTRIC_CHARGE, "mC", "MilliCoulomb", 1.0E-3, 0.0),
    COULOMB(UnitType.ELECTRIC_CHARGE, "C", "Coulomb", 1.0E0, 0.0),
    // Energy
    MILLIJOULE(UnitType.ENERGY, "mJ", "MilliJoule", 1.0E-3, 0.0),
    JOULE(UnitType.ENERGY, "J", "Joule", 1.0E0, 0.0),
    KILOJOULE(UnitType.ENERGY, "kJ", "KiloJoule", 1.0E3, 0.0),
    MEGAJOULE(UnitType.ENERGY, "MJ", "MegaJoule", 1.0E6, 0.0),
    CALORY(UnitType.ENERGY, "cal", "Calory", 4.1868, 0.0),
    KILOCALORY(UnitType.ENERGY, "kcal", "KiloCalory", 4186.8, 0.0),
    WATT_SECOND(UnitType.ENERGY, "W*s", "Watt Second", 1.0E0, 0.0),
    WATT_HOUR(UnitType.ENERGY, "W*h", "Watt Hour", 3600, 0.0),
    KILOWATT_SECOND(UnitType.ENERGY, "kW*s", "KiloWatt Second", 1000, 0.0),
    KILOWATT_HOUR(UnitType.ENERGY, "kW*h", "KiloWatt Hour", 3600000, 0.0),
    // Acceleration
    METER_PER_SQUARE_SECOND(UnitType.ACCELERATION, "m/s²", "Meter per squaresecond", 1.0E0, 0.0),
    INCH_PER_SQUARE_SECOND(UnitType.ACCELERATION, "in/s²", "Inch per squaresecond", 0.0254, 0.0),
    GRAVITY(UnitType.ACCELERATION, "g", "Gravity", 9.80665, 0.0),
    // Pressure
    MILLIPASCAL(UnitType.PRESSURE, "mPa", "MilliPascal", 1.0E-3, 0.0),
    PASCAL(UnitType.PRESSURE, "Pa", "Pascal", 1.0E0, 0.0),
    HECTOPASCAL(UnitType.PRESSURE, "hPa", "HectoPascal", 1.0E2, 0.0),
    KILOPASCAL(UnitType.PRESSURE, "kPa", "KiloPascal", 1.0E3, 0.0),
    BAR(UnitType.PRESSURE, "bar", "Bar", 1.0E5, 0.0),
    MILLIBAR(UnitType.PRESSURE, "mbar", "MilliBar", 1.0E2, 0.0),
    TORR(UnitType.PRESSURE, "Torr", "Torr", 133.322368421, 0.0),
    PSI(UnitType.PRESSURE, "psi", "Psi", 6894.757293178, 0.0),
    // Work
    MILLIWATT(UnitType.WORK, "mW", "MilliWatt", 1.0E-3, 0.0),
    WATT(UnitType.WORK, "W", "Watt", 1.0E0, 0.0),
    KILOWATT(UnitType.WORK, "kW", "KiloWatt", 1.0E3, 0.0),
    MEGAWATT(UnitType.WORK, "MW", "MegaWatt", 1.0E6, 0.0),
    HORSEPOWER(UnitType.WORK, "hp", "HorsePower", 735.49875, 0.0),
    JOULE_PER_SECOND(UnitType.WORK, "J/s", "Joule per second", 1.0E0, 0.0);
   
    private final UnitType UNIT_TYPE;
    private final String UNIT_STRING;
    private final String UNIT_NAME;
    private final double FACTOR;
    private final double OFFSET;
    
    private UnitDescriptor(final UnitType UNIT_TYPE, final String UNIT_STRING, final String UNIT_NAME, final double FACTOR, final double OFFSET)
    {        
        this.UNIT_TYPE = UNIT_TYPE;
        this.UNIT_STRING = UNIT_STRING;
        this.UNIT_NAME = UNIT_NAME;
        this.FACTOR = FACTOR;
        this.OFFSET = OFFSET;
    }
    
    public UnitType getUnitType()
    {
        return this.UNIT_TYPE;
    }

    public String getUnitString()
    {
        return this.UNIT_STRING;
    }

    public String getUnitName()
    {
        return this.UNIT_NAME;
    }

    public double getFactor()
    {
        return this.FACTOR;
    }

    public double getOffset()
    {
        return this.OFFSET;
    }
}
