/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package conversionfield;


/**
 *
 * @author hansolo
 */
public class ConversionField extends javax.swing.JTextField implements java.awt.event.FocusListener
{
    private conversion.Unit unit = null;
    private conversion.UnitType unitType = null;
    private conversion.UnitDescriptor baseUnit = null;
    private int decimals = 3;
    private java.util.regex.Pattern unitStringPattern = null;
    private java.util.regex.Matcher unitStringMatcher = null;
    private final java.util.ArrayList<conversion.UnitDescriptor> AVAILABLE_UNITS = new java.util.ArrayList<conversion.UnitDescriptor>();
    private java.text.DecimalFormat outputFormat = new java.text.DecimalFormat("####0.000", java.text.DecimalFormatSymbols.getInstance(java.util.Locale.ENGLISH));
    

    public ConversionField()
    {
        super();
        init();
    }

    private void init()
    {
        setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        setText("0.0");
        addFocusListener(this);
        setUnitType(conversion.UnitType.LENGTH);
    }

    public java.util.List getAvailableUnits()
    {
        return java.util.Collections.unmodifiableList(this.AVAILABLE_UNITS);
    }
    
    public int getDecimals()
    {
        return this.decimals;
    }

    public void setDecimals(final int DECIMALS)
    {
        this.decimals = DECIMALS;

        final StringBuilder FORMAT_BUILDER = new StringBuilder("####0.");
        for (int i = 0 ; i < decimals ; i++)
        {
            FORMAT_BUILDER.append("0");
        }

        this.outputFormat = new java.text.DecimalFormat(FORMAT_BUILDER.toString(), java.text.DecimalFormatSymbols.getInstance(java.util.Locale.ENGLISH));
        validateInput();
    }

    public conversion.UnitDescriptor getBaseUnit()
    {
        return this.baseUnit;
    }

    public void setBaseUnit(final conversion.UnitDescriptor BASE_UNIT)
    {
        if (BASE_UNIT.getUnitType() != getUnitType())
        {
            throw new IllegalArgumentException("type of units of units has to be the same");
        }        
        this.unit.setBaseUnit(BASE_UNIT);
        this.baseUnit = BASE_UNIT;
        this.unitStringPattern = this.unit.getPattern();
        this.unitStringMatcher = this.unitStringPattern.matcher("");
        validateInput();
    }
    
    public conversion.UnitType getUnitType()
    {
        return this.unitType;
    }

    public void setUnitType(final conversion.UnitType UNIT_TYPE)
    {
        this.unitType = UNIT_TYPE;
        this.unit = new conversion.Unit(UNIT_TYPE);
        this.unitStringPattern = this.unit.getPattern();
        this.unitStringMatcher = this.unitStringPattern.matcher("");
        this.baseUnit = this.unit.getBaseUnit();
        AVAILABLE_UNITS.clear();
        AVAILABLE_UNITS.addAll(this.unit.getUnitTypes().get(this.unit.getUnitType()));
        validateInput();
    }

    public void validateInput()
    {        
        this.unitStringMatcher.reset(getText().trim());
        if (unitStringMatcher.matches())
        {
            for(conversion.UnitDescriptor currentUnit : AVAILABLE_UNITS)
            {
                if (currentUnit.getUnitString().equals(unitStringMatcher.group(2)))
                {
                    setText(outputFormat.format(unit.convertToBaseUnit(Double.parseDouble(unitStringMatcher.group(1)), currentUnit)) + " " + baseUnit.getUnitString());
                    break;
                }
                else
                {
                    setText(outputFormat.format(unit.convertToBaseUnit(Double.parseDouble(unitStringMatcher.group(1)), baseUnit)) + " " + baseUnit.getUnitString());
                }
            }
        }
    }

    // FocusListener methods
    @Override
    public void focusGained(java.awt.event.FocusEvent event)
    {
        
    }

    @Override
    public void focusLost(java.awt.event.FocusEvent event)
    {
        validateInput();
    }

    @Override
    public String toString()
    {
        return "ConversionField";
    }
}
